@@ -11,24 +11,21 @@ import (
 	"strings"
 	"time"
 
-	"github.com/hashicorp/go-version"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
-	. "github.com/onsi/gomega/gstruct"
 
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/api/node/v1beta1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
 	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/klog"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
 	"k8s.io/utils/pointer"
 
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
-	v1 "github.com/openshift/api/config/v1"
 	tunedv1 "github.com/openshift/cluster-node-tuning-operator/pkg/apis/tuned/v1"
 	machineconfigv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 
@@ -37,14 +34,15 @@ import (
 	performancev2 "github.com/openshift-kni/performance-addon-operators/api/v2"
 	testutils "github.com/openshift-kni/performance-addon-operators/functests/utils"
 	testclient "github.com/openshift-kni/performance-addon-operators/functests/utils/client"
+	"github.com/openshift-kni/performance-addon-operators/functests/utils/cluster"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/discovery"
+	testlog "github.com/openshift-kni/performance-addon-operators/functests/utils/log"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/mcps"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/nodes"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/pods"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/profiles"
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components"
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/machineconfig"
-	profilecomponent "github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/profile"
 )
 
 const (
@@ -52,11 +50,19 @@ const (
 	testPollInterval = 2
 )
 
+var RunningOnSingleNode bool
+
 var _ = Describe("[rfe_id:27368][performance]", func() {
 
 	var workerRTNodes []corev1.Node
 	var profile *performancev2.PerformanceProfile
 
+	testutils.BeforeAll(func() {
+		isSNO, err := cluster.IsSingleNode()
+		Expect(err).ToNot(HaveOccurred())
+		RunningOnSingleNode = isSNO
+	})
+
 	BeforeEach(func() {
 		if discovery.Enabled() && testutils.ProfileNotFound {
 			Skip("Discovery mode enabled, performance profile not found")
@@ -64,12 +70,12 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 
 		var err error
 		workerRTNodes, err = nodes.GetByLabels(testutils.NodeSelectorLabels)
-		Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("error looking for node with role %q: %v", testutils.RoleWorkerCNF, err))
+		Expect(err).ToNot(HaveOccurred(), "error looking for node with role %q: %v", testutils.RoleWorkerCNF, err)
 		workerRTNodes, err = nodes.MatchingOptionalSelector(workerRTNodes)
 		Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("error looking for the optional selector: %v", err))
-		Expect(workerRTNodes).ToNot(BeEmpty(), fmt.Sprintf("no nodes with role %q found", testutils.RoleWorkerCNF))
+		Expect(workerRTNodes).ToNot(BeEmpty(), "no nodes with role %q found", testutils.RoleWorkerCNF)
 		profile, err = profiles.GetByNodeLabels(testutils.NodeSelectorLabels)
-		Expect(err).ToNot(HaveOccurred())
+		Expect(err).ToNot(HaveOccurred(), "cannot get profile by node labels %v", testutils.NodeSelectorLabels)
 	})
 
 	// self-tests; these are only vaguely related to performance becase these are enablement conditions, not actual settings.
@@ -90,8 +96,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 				}
 			}
 
-			// Fail
-			Expect(true).To(Reject(), "Performance Addon Operator is not running in a master node")
+			Fail("Performance Addon Operator is not running in a master node")
 		})
 	})
 
@@ -105,7 +110,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 			tuned := &tunedv1.Tuned{}
 			err := testclient.Client.Get(context.TODO(), key, tuned)
-			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object "+tuned.Name)
+			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object %q", tuned.Name)
 
 			Eventually(func() bool {
 				err := testclient.Client.List(context.TODO(), tunedList)
@@ -120,16 +125,17 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 					}
 				}
 				return true
-			}, 120*time.Second, testPollInterval*time.Second).Should(BeTrue(),
-				"tuned CR name owned by a performance profile CR should only be "+tunedExpectedName)
+			}, cluster.ComputeTestTimeout(120*time.Second, RunningOnSingleNode), testPollInterval*time.Second).Should(BeTrue(),
+				"tuned CR name owned by a performance profile CR should only be %q", tunedExpectedName)
 		})
 
 		It("[test_id:37127] Node should point to right tuned profile", func() {
 			for _, node := range workerRTNodes {
 				tuned := tunedForNode(&node)
 				activeProfile, err := pods.ExecCommandOnPod(tuned, []string{"cat", "/etc/tuned/active_profile"})
-				Expect(err).ToNot(HaveOccurred())
-				Expect(strings.TrimSpace(string(activeProfile))).To(Equal(tunedExpectedName))
+				Expect(err).ToNot(HaveOccurred(), "Error getting the tuned active profile")
+				activeProfileName := string(activeProfile)
+				Expect(strings.TrimSpace(activeProfileName)).To(Equal(tunedExpectedName), "active profile name mismatch got %q expected %q", activeProfileName, tunedExpectedName)
 			}
 		})
 	})
@@ -145,7 +151,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 		})
 
-		It("[test_id:32702] Should set CPU isolcpu's kernel argument managed_irq flag", func() {
+		It("Should set CPU isolcpu's kernel argument managed_irq flag", func() {
 			for _, node := range workerRTNodes {
 				cmdline, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"cat", "/proc/cmdline"})
 				Expect(err).ToNot(HaveOccurred())
@@ -196,60 +202,13 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 		})
 
 		It("[test_id:35363][crit:high][vendor:cnf-qe@redhat.com][level:acceptance] stalld daemon is running on the host", func() {
-			By("Checking minimal required cluster version")
-			clusterVersion := ""
-			cv := &v1.ClusterVersion{}
-			key := types.NamespacedName{
-				Name: "version",
-			}
-			err := testclient.Client.Get(context.TODO(), key, cv)
-			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster version object "+key.String())
-			clusterVersion = cv.Status.Desired.Version
-			currentClusterVersion, err := version.NewVersion(clusterVersion)
-			Expect(err).ToNot(HaveOccurred())
-			requiredStalldClusterVersion, err := version.NewVersion("4.7.7")
-			Expect(err).ToNot(HaveOccurred())
-			if strings.Contains(clusterVersion, "ci") || strings.Contains(clusterVersion, "nightly") ||
-				currentClusterVersion.GreaterThanOrEqual(requiredStalldClusterVersion) {
-				for _, node := range workerRTNodes {
-					tuned := tunedForNode(&node)
-					_, err = pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
-					Expect(err).ToNot(HaveOccurred())
-				}
-			} else {
-				klog.Warningf("Stalld is not enable, cluster version: %s", clusterVersion)
-			}
-
-			// this test will both determine if the minimum required kernel version exists in order for stalld to work
-			// and alert in case a wrong kernel version is provided in an openshift release that is being tested.
-			By("Checking minmal required kernel version")
-			stalldMinimumKernelVersionPrefix := "4.18.0-240.22"
-			minKernelPrefix := strings.Split(stalldMinimumKernelVersionPrefix, ".")
-			minSubVersion, err := strconv.Atoi(minKernelPrefix[3])
-			Expect(err).ToNot(HaveOccurred())
+			Skip("until bugs https://bugzilla.redhat.com/show_bug.cgi?id=1912118 and https://bugzilla.redhat.com/show_bug.cgi?id=1903302 fixed")
 			for _, node := range workerRTNodes {
-				cmd := []string{"uname", "-r"}
-				kernel, err := nodes.ExecCommandOnNode(cmd, &node)
-				Expect(err).ToNot(HaveOccurred(), "failed to execute uname")
-				currentKernelPrefix := strings.Split(kernel, ".")
-				currentSubVersion, err := strconv.Atoi(currentKernelPrefix[3])
-				// check if current kernel is a new rt batch stream (higher than 0-240)
-				if err != nil && strings.Contains(currentKernelPrefix[3], "rt") {
-					batchVersionStr := strings.Split(currentKernelPrefix[2], "-")
-					batchVersion, err := strconv.Atoi(batchVersionStr[1])
-					Expect(err).ToNot(HaveOccurred())
-					Expect(batchVersion > 240).Should(BeTrue(), fmt.Sprintf("Current kernel version %s is incompatible to the current oc release", kernel))
-					tuned := tunedForNode(&node)
-					_, err = pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
-					Expect(err).ToNot(HaveOccurred())
-				} else if currentSubVersion >= minSubVersion {
-					tuned := tunedForNode(&node)
-					_, err := pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
-					Expect(err).ToNot(HaveOccurred())
-				}
+				tuned := tunedForNode(&node)
+				_, err := pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
+				Expect(err).ToNot(HaveOccurred())
 			}
 		})
-
 	})
 
 	Context("Additional kernel arguments added from perfomance profile", func() {
@@ -283,7 +242,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			tuned := &tunedv1.Tuned{}
 			err := testclient.Client.Get(context.TODO(), key, tuned)
 			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object "+key.String())
-			validatTunedActiveProfile(workerRTNodes)
+			validateTunedActiveProfile(workerRTNodes)
 			execSysctlOnWorkers(workerRTNodes, sysctlMap)
 		})
 	})
@@ -309,7 +268,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 				hook := hooks["hook"].(map[string]interface{})
 				Expect(hook).ToNot(BeNil())
 				args := hook["args"].([]interface{})
-				Expect(len(args)).To(Equal(2))
+				Expect(len(args)).To(Equal(2), "unexpected arguments: %v", args)
 
 				rpsCPUs, err := components.CPUMaskToCPUSet(args[1].(string))
 				Expect(err).ToNot(HaveOccurred())
@@ -374,105 +333,57 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			tuned := &tunedv1.Tuned{}
 			err := testclient.Client.Get(context.TODO(), key, tuned)
 			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object "+components.ProfileNamePerformance)
-			validatTunedActiveProfile(workerRTNodes)
+			validateTunedActiveProfile(workerRTNodes)
 			execSysctlOnWorkers(workerRTNodes, sysctlMap)
 		})
 	})
 
-	Context("KubeletConfig experimental annotation", func() {
-		var secondMCP *machineconfigv1.MachineConfigPool
-		var secondProfile *performancev2.PerformanceProfile
-		var newRole = "test-annotation"
+	Context("Network device queues adjusted by Tuned", func() {
+		It("[test_id:40308][crit:high][vendor:cnf-qe@redhat.com][level:acceptance] Should be set to the profile's reserved CPUs count ", func() {
+			noDeviceFound := true
+			if profile.Spec.Net != nil {
+				reservedSet, err := cpuset.Parse(string(*profile.Spec.CPU.Reserved))
+				Expect(err).ToNot(HaveOccurred())
+				reserveCPUsCount := reservedSet.Size()
+				if profile.Spec.Net.UserLevelNetworking != nil && *profile.Spec.Net.UserLevelNetworking && len(profile.Spec.Net.Devices) == 0 {
+					By("To all non virtual network devices when no devices are specified under profile.Spec.Net.Devices")
+					for _, node := range workerRTNodes {
 
-		BeforeEach(func() {
-			newLabel := fmt.Sprintf("%s/%s", testutils.LabelRole, newRole)
+						cmdGetPhysicalDevices := []string{"find", "/sys/class/net", "-type", "l", "-not", "-lname", "*virtual*", "-printf", "%f "}
+						By(fmt.Sprintf("getting a list of physical network devices: %v", cmdGetPhysicalDevices))
+						tuned := tunedForNode(&node)
+						phyDevs, err := pods.ExecCommandOnPod(tuned, cmdGetPhysicalDevices)
+						Expect(err).ToNot(HaveOccurred())
 
-			reserved := performancev2.CPUSet("0")
-			isolated := performancev2.CPUSet("1-3")
+						for _, d := range strings.Split(string(phyDevs), " ") {
+							if d == "" {
+								continue
+							}
+							// See if the device 'd' supports querying the channels.
+							_, err := pods.ExecCommandOnPod(tuned, []string{"ethtool", "-l", d})
+							if err == nil {
+								cmdCombinedChannelsCurrent := []string{"bash", "-c",
+									fmt.Sprintf("ethtool -l %s | sed -n '/Current hardware settings:/,/Combined:/{s/^Combined:\\s*//p}'", d)}
 
-			secondProfile = &performancev2.PerformanceProfile{
-				TypeMeta: metav1.TypeMeta{
-					Kind:       "PerformanceProfile",
-					APIVersion: performancev2.GroupVersion.String(),
-				},
-				ObjectMeta: metav1.ObjectMeta{
-					Name: "test-annotation",
-					Annotations: map[string]string{
-						"kubeletconfig.experimental": `{"systemReserved": {"memory": "256Mi"}, "kubeReserved": {"memory": "256Mi"}}`,
-					},
-				},
-				Spec: performancev2.PerformanceProfileSpec{
-					CPU: &performancev2.CPU{
-						Reserved: &reserved,
-						Isolated: &isolated,
-					},
-					NodeSelector: map[string]string{newLabel: ""},
-					RealTimeKernel: &performancev2.RealTimeKernel{
-						Enabled: pointer.BoolPtr(true),
-					},
-					NUMA: &performancev2.NUMA{
-						TopologyPolicy: pointer.StringPtr("restricted"),
-					},
-				},
-			}
-			Expect(testclient.Client.Create(context.TODO(), secondProfile)).ToNot(HaveOccurred())
+								By(fmt.Sprintf("using physical network device %s for testing", d))
+								out, err := pods.ExecCommandOnPod(tuned, cmdCombinedChannelsCurrent)
+								Expect(err).NotTo(HaveOccurred())
+								channelCurrentCombined, err := strconv.Atoi(strings.TrimSpace(string(out)))
+								if err != nil {
+									testlog.Warningf(fmt.Sprintf("unable to retrieve current multi-purpose channels hardware settings for device %s on %s; skipping test: %v",
+										d, node.Name, err))
+								} else {
+									Expect(err).ToNot(HaveOccurred())
+									Expect(channelCurrentCombined).To(Equal(reserveCPUsCount), " Channel current combine count does not match the count of reserverd CPUs")
+								}
+								noDeviceFound = false
+							}
+						}
 
-			machineConfigSelector := profilecomponent.GetMachineConfigLabel(secondProfile)
-			secondMCP = &machineconfigv1.MachineConfigPool{
-				ObjectMeta: metav1.ObjectMeta{
-					Name: "test-annotation",
-					Labels: map[string]string{
-						machineconfigv1.MachineConfigRoleLabelKey: newRole,
-					},
-				},
-				Spec: machineconfigv1.MachineConfigPoolSpec{
-					MachineConfigSelector: &metav1.LabelSelector{
-						MatchLabels: machineConfigSelector,
-					},
-					NodeSelector: &metav1.LabelSelector{
-						MatchLabels: map[string]string{
-							newLabel: "",
-						},
-					},
-				},
-			}
-
-			Expect(testclient.Client.Create(context.TODO(), secondMCP)).ToNot(HaveOccurred())
-		})
-
-		It("should override system-reserved memory", func() {
-			var kubeletConfig *machineconfigv1.KubeletConfig
-
-			Eventually(func() error {
-				By("Getting that new KubeletConfig")
-				configKey := types.NamespacedName{
-					Name:      components.GetComponentName(secondProfile.Name, components.ComponentNamePrefix),
-					Namespace: metav1.NamespaceNone,
-				}
-				kubeletConfig = &machineconfigv1.KubeletConfig{}
-				if err := testclient.GetWithRetry(context.TODO(), configKey, kubeletConfig); err != nil {
-					klog.Warningf("Failed to get the KubeletConfig %q", configKey.Name)
-					return err
-				}
-
-				return nil
-			}, time.Minute, 5*time.Second).Should(BeNil())
-
-			kubeletConfigString := string(kubeletConfig.Spec.KubeletConfig.Raw)
-			Expect(kubeletConfigString).To(ContainSubstring(`"kubeReserved":{"memory":"256Mi"}`))
-			Expect(kubeletConfigString).To(ContainSubstring(`"systemReserved":{"memory":"256Mi"}`))
-		})
-
-		AfterEach(func() {
-			if secondProfile != nil {
-				if err := testclient.Client.Delete(context.TODO(), secondProfile); err != nil {
-					klog.Warningf("failed to delete the performance profile %q: %v", secondProfile.Name, err)
-				}
-			}
-
-			if secondMCP != nil {
-				if err := testclient.Client.Delete(context.TODO(), secondMCP); err != nil {
-					klog.Warningf("failed to delete the machine config pool %q: %v", secondMCP.Name, err)
+						if noDeviceFound {
+							Skip(fmt.Sprintf("no network devices supporting querying channels found on node %s; skipping test", node.Name))
+						}
+					}
 				}
 			}
 		})
@@ -516,7 +427,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			By("Checking that new KubeletConfig, MachineConfig and RuntimeClass created")
 			configKey := types.NamespacedName{
 				Name:      components.GetComponentName(secondProfile.Name, components.ComponentNamePrefix),
-				Namespace: components.NamespaceNodeTuningOperator,
+				Namespace: metav1.NamespaceNone,
 			}
 			kubeletConfig := &machineconfigv1.KubeletConfig{}
 			err := testclient.GetWithRetry(context.TODO(), configKey, kubeletConfig)
@@ -524,16 +435,20 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			Expect(kubeletConfig.Spec.MachineConfigPoolSelector.MatchLabels[machineconfigv1.MachineConfigRoleLabelKey]).Should(Equal(newRole))
 			Expect(kubeletConfig.Spec.KubeletConfig.Raw).Should(ContainSubstring("restricted"), "Can't find value in KubeletConfig")
 
-			machineConfig := &machineconfigv1.MachineConfig{}
-			err = testclient.GetWithRetry(context.TODO(), configKey, machineConfig)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find MachineConfig object %s", configKey.Name))
-			Expect(machineConfig.Labels[machineconfigv1.MachineConfigRoleLabelKey]).Should(Equal(newRole))
-
 			runtimeClass := &v1beta1.RuntimeClass{}
 			err = testclient.GetWithRetry(context.TODO(), configKey, runtimeClass)
 			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find RuntimeClass profile object %s", runtimeClass.Name))
 			Expect(runtimeClass.Handler).Should(Equal(machineconfig.HighPerformanceRuntime))
 
+			machineConfigKey := types.NamespacedName{
+				Name:      machineconfig.GetMachineConfigName(secondProfile),
+				Namespace: metav1.NamespaceNone,
+			}
+			machineConfig := &machineconfigv1.MachineConfig{}
+			err = testclient.GetWithRetry(context.TODO(), machineConfigKey, machineConfig)
+			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find MachineConfig object %s", configKey.Name))
+			Expect(machineConfig.Labels[machineconfigv1.MachineConfigRoleLabelKey]).Should(Equal(newRole))
+
 			By("Checking that new Tuned profile created")
 			tunedKey := types.NamespacedName{
 				Name:      components.GetComponentName(secondProfile.Name, components.ProfileNamePerformance),
@@ -564,7 +479,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}, 30, 5).Should(Equal(corev1.ConditionFalse))
 
 			Expect(testclient.Client.Get(context.TODO(), configKey, kubeletConfig)).To(HaveOccurred(), fmt.Sprintf("KubeletConfig %s should be removed", configKey.Name))
-			Expect(testclient.Client.Get(context.TODO(), configKey, machineConfig)).To(HaveOccurred(), fmt.Sprintf("MachineConfig %s should be removed", configKey.Name))
+			Expect(testclient.Client.Get(context.TODO(), machineConfigKey, machineConfig)).To(HaveOccurred(), fmt.Sprintf("MachineConfig %s should be removed", configKey.Name))
 			Expect(testclient.Client.Get(context.TODO(), configKey, runtimeClass)).To(HaveOccurred(), fmt.Sprintf("RuntimeClass %s should be removed", configKey.Name))
 			Expect(testclient.Client.Get(context.TODO(), tunedKey, tunedProfile)).To(HaveOccurred(), fmt.Sprintf("Tuned profile object %s should be removed", tunedKey.Name))
 
@@ -575,7 +490,12 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 			err = testclient.GetWithRetry(context.TODO(), initialKey, &machineconfigv1.KubeletConfig{})
 			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find KubeletConfig object %s", initialKey.Name))
-			err = testclient.GetWithRetry(context.TODO(), initialKey, &machineconfigv1.MachineConfig{})
+
+			initialMachineConfigKey := types.NamespacedName{
+				Name:      machineconfig.GetMachineConfigName(profile),
+				Namespace: metav1.NamespaceNone,
+			}
+			err = testclient.GetWithRetry(context.TODO(), initialMachineConfigKey, &machineconfigv1.MachineConfig{})
 			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find MachineConfig object %s", initialKey.Name))
 		})
 	})
@@ -702,6 +622,197 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			verifyV2V1()
 		})
 	})
+
+	Context("Validation webhook", func() {
+		BeforeEach(func() {
+			if discovery.Enabled() {
+				Skip("Discovery mode enabled, test skipped because it creates incorrect profiles")
+			}
+		})
+
+		validateObject := func(obj runtime.Object, message string) {
+			err := testclient.Client.Create(context.TODO(), obj)
+			Expect(err).To(HaveOccurred(), "expected the validation error")
+			Expect(err.Error()).To(ContainSubstring(message))
+		}
+
+		Context("with API version v1alpha1 profile", func() {
+			var v1alpha1Profile *performancev1alpha1.PerformanceProfile
+
+			BeforeEach(func() {
+				v1alpha1Profile = &performancev1alpha1.PerformanceProfile{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "PerformanceProfile",
+						APIVersion: performancev1alpha1.GroupVersion.String(),
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "v1alpha1-profile",
+					},
+					Spec: performancev1alpha1.PerformanceProfileSpec{
+						RealTimeKernel: &performancev1alpha1.RealTimeKernel{
+							Enabled: pointer.BoolPtr(true),
+						},
+						NodeSelector: map[string]string{"v1alpha1/v1alpha1": "v1alpha1"},
+						NUMA: &performancev1alpha1.NUMA{
+							TopologyPolicy: pointer.StringPtr("restricted"),
+						},
+					},
+				}
+			})
+
+			It("should reject the creation of the profile with overlapping CPUs", func() {
+				reserved := performancev1alpha1.CPUSet("0-3")
+				isolated := performancev1alpha1.CPUSet("0-7")
+
+				v1alpha1Profile.Spec.CPU = &performancev1alpha1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1alpha1Profile, "reserved and isolated cpus overlap")
+			})
+
+			It("should reject the creation of the profile with no isolated CPUs", func() {
+				reserved := performancev1alpha1.CPUSet("0-3")
+				isolated := performancev1alpha1.CPUSet("")
+
+				v1alpha1Profile.Spec.CPU = &performancev1alpha1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1alpha1Profile, "isolated CPUs can not be empty")
+			})
+
+			It("should reject the creation of the profile with the node selector that already in use", func() {
+				reserved := performancev1alpha1.CPUSet("0,1")
+				isolated := performancev1alpha1.CPUSet("2,3")
+
+				v1alpha1Profile.Spec.CPU = &performancev1alpha1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				v1alpha1Profile.Spec.NodeSelector = testutils.NodeSelectorLabels
+				validateObject(v1alpha1Profile, "the profile has the same node selector as the performance profile")
+			})
+		})
+
+		Context("with API version v1 profile", func() {
+			var v1Profile *performancev1.PerformanceProfile
+
+			BeforeEach(func() {
+				v1Profile = &performancev1.PerformanceProfile{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "PerformanceProfile",
+						APIVersion: performancev1.GroupVersion.String(),
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "v1-profile",
+					},
+					Spec: performancev1.PerformanceProfileSpec{
+						RealTimeKernel: &performancev1.RealTimeKernel{
+							Enabled: pointer.BoolPtr(true),
+						},
+						NodeSelector: map[string]string{"v1/v1": "v1"},
+						NUMA: &performancev1.NUMA{
+							TopologyPolicy: pointer.StringPtr("restricted"),
+						},
+					},
+				}
+			})
+
+			It("should reject the creation of the profile with overlapping CPUs", func() {
+				reserved := performancev1.CPUSet("0-3")
+				isolated := performancev1.CPUSet("0-7")
+
+				v1Profile.Spec.CPU = &performancev1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1Profile, "reserved and isolated cpus overlap")
+			})
+
+			It("should reject the creation of the profile with no isolated CPUs", func() {
+				reserved := performancev1.CPUSet("0-3")
+				isolated := performancev1.CPUSet("")
+
+				v1Profile.Spec.CPU = &performancev1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1Profile, "isolated CPUs can not be empty")
+			})
+
+			It("should reject the creation of the profile with the node selector that already in use", func() {
+				reserved := performancev1.CPUSet("0,1")
+				isolated := performancev1.CPUSet("2,3")
+
+				v1Profile.Spec.CPU = &performancev1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				v1Profile.Spec.NodeSelector = testutils.NodeSelectorLabels
+				validateObject(v1Profile, "the profile has the same node selector as the performance profile")
+			})
+		})
+
+		Context("with profile version v2", func() {
+			var v2Profile *performancev2.PerformanceProfile
+
+			BeforeEach(func() {
+				v2Profile = &performancev2.PerformanceProfile{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "PerformanceProfile",
+						APIVersion: performancev2.GroupVersion.String(),
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "v2-profile",
+					},
+					Spec: performancev2.PerformanceProfileSpec{
+						RealTimeKernel: &performancev2.RealTimeKernel{
+							Enabled: pointer.BoolPtr(true),
+						},
+						NodeSelector: map[string]string{"v2/v2": "v2"},
+						NUMA: &performancev2.NUMA{
+							TopologyPolicy: pointer.StringPtr("restricted"),
+						},
+					},
+				}
+			})
+
+			It("should reject the creation of the profile with overlapping CPUs", func() {
+				reserved := performancev2.CPUSet("0-3")
+				isolated := performancev2.CPUSet("0-7")
+
+				v2Profile.Spec.CPU = &performancev2.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v2Profile, "reserved and isolated cpus overlap")
+			})
+
+			It("should reject the creation of the profile with no isolated CPUs", func() {
+				reserved := performancev2.CPUSet("0-3")
+				isolated := performancev2.CPUSet("")
+
+				v2Profile.Spec.CPU = &performancev2.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v2Profile, "isolated CPUs can not be empty")
+			})
+
+			It("should reject the creation of the profile with the node selector that already in use", func() {
+				reserved := performancev2.CPUSet("0,1")
+				isolated := performancev2.CPUSet("2,3")
+
+				v2Profile.Spec.CPU = &performancev2.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				v2Profile.Spec.NodeSelector = testutils.NodeSelectorLabels
+				validateObject(v2Profile, "the profile has the same node selector as the performance profile")
+			})
+		})
+	})
 })
 
 func verifyV1alpha1Conversion(v1alpha1Profile *performancev1alpha1.PerformanceProfile, v1Profile *performancev1.PerformanceProfile) error {
@@ -979,13 +1090,13 @@ func execSysctlOnWorkers(workerNodes []corev1.Node, sysctlMap map[string]string)
 			By(fmt.Sprintf("executing the command \"sysctl -n %s\"", param))
 			out, err = nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"sysctl", "-n", param})
 			Expect(err).ToNot(HaveOccurred())
-			Expect(strings.TrimSpace(string(out))).Should(Equal(expected), fmt.Sprintf("parameter %s value is not %s.", param, expected))
+			Expect(strings.TrimSpace(string(out))).Should(Equal(expected), "parameter %s value is not %s.", param, expected)
 		}
 	}
 }
 
 // execute sysctl command inside container in a tuned pod
-func validatTunedActiveProfile(nodes []corev1.Node) {
+func validateTunedActiveProfile(nodes []corev1.Node) {
 	var err error
 	var out []byte
 	activeProfileName := components.GetComponentName(testutils.PerformanceProfileName, components.ProfileNamePerformance)
@@ -997,7 +1108,7 @@ func validatTunedActiveProfile(nodes []corev1.Node) {
 
 	for _, t := range tunedList.Items {
 		if len(t.Spec.Profile) > 0 && t.Spec.Profile[0].Data != nil && strings.Contains(*t.Spec.Profile[0].Data, fmt.Sprintf("include=%s", activeProfileName)) {
-			klog.Warning(fmt.Sprintf("PAO tuned profile amended by '%s' profile, test may fail", t.Name))
+			testlog.Warning(fmt.Sprintf("PAO tuned profile amended by '%s' profile, test may fail", t.Name))
 			if t.Spec.Profile[0].Name != nil {
 				activeProfileName = *t.Spec.Profile[0].Name
 			}
@@ -1011,7 +1122,7 @@ func validatTunedActiveProfile(nodes []corev1.Node) {
 		Eventually(func() string {
 			out, err = pods.ExecCommandOnPod(tuned, []string{"cat", "/etc/tuned/active_profile"})
 			return strings.TrimSpace(string(out))
-		}, testTimeout*time.Second, testPollInterval*time.Second).Should(Equal(activeProfileName),
+		}, cluster.ComputeTestTimeout(testTimeout*time.Second, RunningOnSingleNode), testPollInterval*time.Second).Should(Equal(activeProfileName),
 			fmt.Sprintf("active_profile is not set to %s. %v", activeProfileName, err))
 	}
 }
@@ -1040,7 +1151,7 @@ func tunedForNode(node *corev1.Node) *corev1.Pod {
 		}
 		return true
 
-	}, testTimeout*time.Second, testPollInterval*time.Second).Should(BeTrue(),
+	}, cluster.ComputeTestTimeout(testTimeout*time.Second, RunningOnSingleNode), testPollInterval*time.Second).Should(BeTrue(),
 		"there should be one tuned daemon per node")
 
 	return &tunedList.Items[0]
